<?php

// phpcs:disable Drupal.Classes.FullyQualifiedNamespace.UseStatementMissing

namespace Drupal\sql_tools\Drush\Commands;

use Consolidation\OutputFormatters\StructuredData\PropertyList;
use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drupal\sql_tools\Utils\ByteSizeMarkup;
use Drush\Attributes as CLI;
use Drush\Boot\DrupalBootLevels;
use Drush\Commands\DrushCommands;
use Drush\Sql\SqlBase;

/**
 * Utility to provide the database size or table sizes.
 */
class SqlToolsDrushCommands extends DrushCommands {

  /**
   * SQL query to determine the size of a database.
   *
   * @var string
   */
  const SQL_QUERY_DATABASE_SIZE = 'SELECT SUM(data_length + index_length) AS `size` FROM information_schema.TABLES WHERE TABLE_SCHEMA = :database_name GROUP BY TABLE_SCHEMA';

  /**
   * SQL query to determine the size of database tables for a database.
   *
   * @var string
   */
  const SQL_QUERY_TABLE_SIZE = 'SELECT TABLE_NAME, (data_length + index_length) AS `size` FROM information_schema.TABLES WHERE TABLE_SCHEMA = :database_name';

  /**
   * SQL query to optimize a database table.
   *
   * @var string
   */
  const SQL_QUERY_OPTIMIZE_TABLE = 'OPTIMIZE TABLE :table_name';

  /**
   * Query the size of a database.
   */
  #[CLI\Command(name: 'sql:dbsize')]
  #[CLI\OptionsetSql]
  #[CLI\Bootstrap(DrupalBootLevels::DATABASE)]
  #[CLI\Option(
    name: 'human',
    description: 'Report the size with units (KB, MB, GB, etc).',
    suggestedValues: [TRUE, FALSE]),
  ]
  public function dbSize(
    $options = [
      'result-file' => NULL,
      'extra' => self::REQ,
      'format' => 'table',
      'human' => FALSE,
    ],
  ) {
    $sql = SqlBase::create($options);
    $database = $sql->getDbSpec()['database'];
    $query = str_replace(':database_name', "'{$database}'", self::SQL_QUERY_DATABASE_SIZE);
    $result = $sql->query($query, NULL, $options['result-file']);

    if ($result) {
      $size = (int) trim($sql->getProcess()->getOutput());

      // @todo Format the size with a renderer.
      if ($options['human']) {
        $size = ByteSizeMarkup::create($size);
      }
      return new PropertyList(['size' => $size]);
    }
    $this->io()->error('Unable to determine database size.');
  }

  /**
   * Query the size of each table.
   */
  #[CLI\Command(name: 'sql:tablesize')]
  #[CLI\OptionsetSql]
  #[CLI\Bootstrap(DrupalBootLevels::DATABASE)]
  #[CLI\FieldLabels(labels: ['table' => 'Table', 'size' => 'Size'])]
  #[CLI\FilterDefaultField(field: 'table')]
  #[CLI\Option('sort', 'Sort by table name or size', ['table', 'size'])]
  #[CLI\Option('sort-order', 'Sort by ascending or descending', ['asc', 'desc'])]
  #[CLI\Option(
    name: 'human',
    description: 'Report the size with units (KB, MB, GB, etc).',
    suggestedValues: [TRUE, FALSE]),
  ]
  public function tableSize(
    $options = [
      'result-file' => NULL,
      'extra' => self::REQ,
      'format' => 'table',
      'sort' => 'table',
      'sort-order' => 'asc',
      'human' => FALSE,
    ],
  ) {
    $sql = SqlBase::create($options);
    $database = $sql->getDbSpec()['database'];
    $query = str_replace(':database_name', "'{$database}'", self::SQL_QUERY_TABLE_SIZE);
    $result = $sql->query($query, NULL, $options['result-file']);

    if ($result) {
      $data = [];
      foreach (explode("\n", trim($sql->getProcess()->getOutput())) as $line) {
        [$table, $size] = explode("\t", $line);
        $data[] = [
          'table' => $table,
          'size' => $size,
        ];
      }

      $sort = $options['sort'];
      usort($data, function ($aaa, $bbb) use ($sort) {
        return $aaa[$sort] <=> $bbb[$sort];
      });
      if ($options['sort-order'] == 'desc') {
        $data = array_reverse($data);
      }
      if ($options['human']) {
        array_walk($data, fn (&$row) => $row['size'] = ByteSizeMarkup::create($row['size']));
      }

      return new RowsOfFields($data);
    }
    $this->io()->error('Unable to determine table sizes.');
  }

  /**
   * Query the size of each table.
   */
  #[CLI\Command(name: 'sql:optimize')]
  #[CLI\OptionsetSql]
  #[CLI\Bootstrap(DrupalBootLevels::DATABASE)]
  public function optimizeTables(
    $options = [
      'extra' => self::REQ,
      'result-file' => NULL,
    ],
  ) {
    $sql = SqlBase::create($options);
    $tables = $sql->listTables();

    foreach ($tables as $table) {
      $query = str_replace(':table_name', "{$table}", self::SQL_QUERY_OPTIMIZE_TABLE);
      $result = $sql->query($query, NULL, $options['result-file']);
      if ($result) {
        $this->io()->success(str_replace(':table', $table, 'Optimized table :table'));
      }
      else {
        $this->io()->warning(str_replace(':table', $table, 'Failed to optimize table :table'));
      }
    }
  }

}
