<?php

declare (strict_types=1);

namespace Drupal\sql_tools\Utils;

use Drupal\Component\Utility\Bytes;

/**
 * Rudimentary implementation of \Drupal\Core\StringTranslation\ByteSizeMarkup.
 */
class ByteSizeMarkup {

  /**
   * This class should not be instantiated.
   */
  private function __construct() {
  }

  /**
   * Gets the string with units for the provided size.
   *
   * @return string
   *   The size with units.
   *
   * @throws \LogicException
   *   Thrown when an invalid unit size is used.
   */
  public static function create(float|int $size) : string {
    $absolute_size = abs($size);
    if ($absolute_size < Bytes::KILOBYTE) {
      return ($size === 1)
            ? dt('1 byte')
            : dt('@count bytes', ['@count' => $size]);
    }

    // Create a multiplier to preserve the sign of $size.
    $sign = $absolute_size / $size;
    foreach ([
      'KB',
      'MB',
      'GB',
      'TB',
      'PB',
      'EB',
      'ZB',
      'YB',
    ] as $unit) {
      $absolute_size /= Bytes::KILOBYTE;
      $rounded_size = round($absolute_size, 2);
      if ($rounded_size < Bytes::KILOBYTE) {
        break;
      }
    }
    $args = [
      '@size' => $rounded_size * $sign,
    ];
    // At this point $markup must be set.
    return match ($unit) {
      'KB' => dt('@size KB', $args),
          'MB' => dt('@size MB', $args),
          'GB' => dt('@size GB', $args),
          'TB' => dt('@size TB', $args),
          'PB' => dt('@size PB', $args),
          'EB' => dt('@size EB', $args),
          'ZB' => dt('@size ZB', $args),
          'YB' => dt('@size YB', $args),
          default => throw new \LogicException("Unexpected unit value"),
    };
  }

}
